@@ -4,6 +4,14 @@ class LogEntry:
         self.msg = msg
         self.level = level
 
+    def __eq__(self, other):
+        if isinstance(other, LogEntry):
+            return self.__dict__ == other.__dict__
+        return False
+
+    def __repr__(self):
+        return "LogEntry({}, {})".format(self.msg, self.level)
+
 
 class Log:
     """
@@ -24,6 +32,15 @@ class Log:
         """
         self(txt, "info")
 
+    def alert(self, txt):
+        """
+            Log with level alert. Alerts have the same urgency as info, but
+            signals to interctive tools that the user's attention should be
+            drawn to the output even if they're not currently looking at the
+            event log.
+        """
+        self(txt, "alert")
+
     def warn(self, txt):
         """
             Log with level warn.
@@ -41,4 +58,4 @@ class Log:
 
 
 def log_tier(level):
-    return dict(error=0, warn=1, info=2, debug=3).get(level)
+    return dict(error=0, warn=1, info=2, alert=2, debug=3).get(level)
