@@ -1,9 +1,4 @@
 from io import BytesIO
-import tempfile
-import os
-import time
-import shutil
-from contextlib import contextmanager
 
 from mitmproxy.utils import data
 from mitmproxy.net import tcp
@@ -13,18 +8,6 @@ from mitmproxy.net import http
 test_data = data.Data(__name__).push("../../test/")
 
 
-@contextmanager
-def tmpdir(*args, **kwargs):
-    orig_workdir = os.getcwd()
-    temp_workdir = tempfile.mkdtemp(*args, **kwargs)
-    os.chdir(temp_workdir)
-
-    yield temp_workdir
-
-    os.chdir(orig_workdir)
-    shutil.rmtree(temp_workdir)
-
-
 def treader(bytes):
     """
         Construct a tcp.Read object from bytes.
@@ -47,7 +30,9 @@ def treq(**kwargs):
         path=b"/path",
         http_version=b"HTTP/1.1",
         headers=http.Headers(((b"header", b"qvalue"), (b"content-length", b"7"))),
-        content=b"content"
+        content=b"content",
+        timestamp_start=946681200,
+        timestamp_end=946681201,
     )
     default.update(kwargs)
     return http.Request(**default)
@@ -64,8 +49,8 @@ def tresp(**kwargs):
         reason=b"OK",
         headers=http.Headers(((b"header-response", b"svalue"), (b"content-length", b"7"))),
         content=b"message",
-        timestamp_start=time.time(),
-        timestamp_end=time.time(),
+        timestamp_start=946681202,
+        timestamp_end=946681203,
     )
     default.update(kwargs)
     return http.Response(**default)
