@@ -1,24 +1,29 @@
 import abc
 import copy
-from typing import Any
-from typing import Callable
-from typing import Container
-from typing import Iterable
-from typing import Optional
-from typing import Sequence
-from typing import Tuple
-
+import os
+import typing
 import urwid
-from mitmproxy.tools.console import common
-from mitmproxy.tools.console import signals
 
-FOOTER = [
-    ('heading_key', "enter"), ":edit ",
-    ('heading_key', "q"), ":back ",
-]
-FOOTER_EDITING = [
-    ('heading_key', "esc"), ":stop editing ",
-]
+from mitmproxy.utils import strutils
+from mitmproxy import exceptions
+from mitmproxy.tools.console import signals
+from mitmproxy.tools.console import layoutwidget
+import mitmproxy.tools.console.master # noqa
+
+
+def read_file(filename: str, escaped: bool) -> typing.AnyStr:
+    filename = os.path.expanduser(filename)
+    try:
+        with open(filename, "r" if escaped else "rb") as f:
+            d = f.read()
+    except IOError as v:
+        raise exceptions.CommandError(v)
+    if escaped:
+        try:
+            d = strutils.escaped_str_to_bytes(d)
+        except ValueError:
+            raise exceptions.CommandError("Invalid Python-style string encoding.")
+    return d
 
 
 class Cell(urwid.WidgetWrap):
@@ -34,7 +39,7 @@ class Cell(urwid.WidgetWrap):
 
 
 class Column(metaclass=abc.ABCMeta):
-    subeditor = None
+    subeditor = None  # type: urwid.Edit
 
     def __init__(self, heading):
         self.heading = heading
@@ -48,27 +53,28 @@ class Column(metaclass=abc.ABCMeta):
         pass
 
     @abc.abstractmethod
-    def blank(self) -> Any:
+    def blank(self) -> typing.Any:
         pass
 
-    def keypress(self, key: str, editor: "GridEditor") -> Optional[str]:
+    def keypress(self, key: str, editor: "GridEditor") -> typing.Optional[str]:
         return key
 
 
 class GridRow(urwid.WidgetWrap):
+
     def __init__(
             self,
-            focused: Optional[int],
+            focused: typing.Optional[int],
             editing: bool,
             editor: "GridEditor",
-            values: Tuple[Iterable[bytes], Container[int]]
-    ):
+            values: typing.Tuple[typing.Iterable[bytes], typing.Container[int]]
+    ) -> None:
         self.focused = focused
         self.editor = editor
-        self.edit_col = None  # type: Optional[Cell]
+        self.edit_col = None  # type: typing.Optional[Cell]
 
         errors = values[1]
-        self.fields = []
+        self.fields = []  # type: typing.Sequence[typing.Any]
         for i, v in enumerate(values[0]):
             if focused == i and editing:
                 self.edit_col = self.editor.columns[i].Edit(v)
@@ -114,14 +120,14 @@ class GridWalker(urwid.ListWalker):
 
     def __init__(
             self,
-            lst: Iterable[list],
+            lst: typing.Iterable[list],
             editor: "GridEditor"
-    ):
-        self.lst = [(i, set()) for i in lst]
+    ) -> None:
+        self.lst = [(i, set()) for i in lst]  # type: typing.Sequence[typing.Tuple[typing.Any, typing.Set]]
         self.editor = editor
         self.focus = 0
         self.focus_col = 0
-        self.edit_row = None  # type: Optional[GridRow]
+        self.edit_row = None  # type: typing.Optional[GridRow]
 
     def _modified(self):
         self.editor.show_empty_msg()
@@ -182,12 +188,10 @@ class GridWalker(urwid.ListWalker):
             self.edit_row = GridRow(
                 self.focus_col, True, self.editor, self.lst[self.focus]
             )
-            self.editor.master.loop.widget.footer.update(FOOTER_EDITING)
             self._modified()
 
     def stop_edit(self):
         if self.edit_row:
-            self.editor.master.loop.widget.footer.update(FOOTER)
             try:
                 val = self.edit_row.edit_col.get_data()
             except ValueError:
@@ -247,23 +251,26 @@ class GridListBox(urwid.ListBox):
 
 
 FIRST_WIDTH_MAX = 40
-FIRST_WIDTH_MIN = 20
 
 
-class GridEditor(urwid.WidgetWrap):
-    title = None  # type: str
-    columns = None  # type: Sequence[Column]
+class BaseGridEditor(urwid.WidgetWrap):
+    title = ""
+    keyctx = "grideditor"
 
     def __init__(
             self,
-            master: "mitmproxy.console.master.ConsoleMaster",
-            value: Any,
-            callback: Callable[..., None],
+            master: "mitmproxy.tools.console.master.ConsoleMaster",
+            title,
+            columns,
+            value: typing.Any,
+            callback: typing.Callable[..., None],
             *cb_args,
             **cb_kwargs
-    ):
+    ) -> None:
         value = self.data_in(copy.deepcopy(value))
         self.master = master
+        self.title = title
+        self.columns = columns
         self.value = value
         self.callback = callback
         self.cb_args = cb_args
@@ -276,33 +283,35 @@ class GridEditor(urwid.WidgetWrap):
                 first_width = max(len(r), first_width)
         self.first_width = min(first_width, FIRST_WIDTH_MAX)
 
-        title = urwid.Text(self.title)
-        title = urwid.Padding(title, align="left", width=("relative", 100))
-        title = urwid.AttrWrap(title, "heading")
-
-        headings = []
-        for i, col in enumerate(self.columns):
-            c = urwid.Text(col.heading)
-            if i == 0 and len(self.columns) > 1:
-                headings.append(("fixed", first_width + 2, c))
-            else:
-                headings.append(c)
-        h = urwid.Columns(
-            headings,
-            dividechars=2
-        )
-        h = urwid.AttrWrap(h, "heading")
+        h = None
+        if any(col.heading for col in self.columns):
+            headings = []
+            for i, col in enumerate(self.columns):
+                c = urwid.Text(col.heading)
+                if i == 0 and len(self.columns) > 1:
+                    headings.append(("fixed", first_width + 2, c))
+                else:
+                    headings.append(c)
+            h = urwid.Columns(
+                headings,
+                dividechars=2
+            )
+            h = urwid.AttrWrap(h, "heading")
 
         self.walker = GridWalker(self.value, self)
         self.lb = GridListBox(self.walker)
-        w = urwid.Frame(
-            self.lb,
-            header=urwid.Pile([title, h])
-        )
+        w = urwid.Frame(self.lb, header=h)
+
         super().__init__(w)
-        self.master.loop.widget.footer.update("")
         self.show_empty_msg()
 
+    def layout_popping(self):
+        res = []
+        for i in self.walker.lst:
+            if not i[1] and any([x for x in i[0]]):
+                res.append(i[0])
+        self.callback(self.data_out(res), *self.cb_args, **self.cb_kwargs)
+
     def show_empty_msg(self):
         if self.walker.lst:
             self._w.set_footer(None)
@@ -310,9 +319,9 @@ class GridEditor(urwid.WidgetWrap):
             self._w.set_footer(
                 urwid.Text(
                     [
-                        ("highlight", "No values. Press "),
-                        ("key", "a"),
-                        ("highlight", " to add some."),
+                        ("highlight", "No values - you should add some. Press "),
+                        ("key", "?"),
+                        ("highlight", " for help."),
                     ]
                 )
             )
@@ -322,7 +331,7 @@ class GridEditor(urwid.WidgetWrap):
 
     def keypress(self, size, key):
         if self.walker.edit_row:
-            if key in ["esc"]:
+            if key == "esc":
                 self.walker.stop_edit()
             elif key == "tab":
                 pf, pfc = self.walker.focus, self.walker.focus_col
@@ -333,81 +342,134 @@ class GridEditor(urwid.WidgetWrap):
                 self._w.keypress(size, key)
             return None
 
-        key = common.shortcuts(key)
         column = self.columns[self.walker.focus_col]
-        if key in ["q", "esc"]:
-            res = []
-            for i in self.walker.lst:
-                if not i[1] and any([x for x in i[0]]):
-                    res.append(i[0])
-            self.callback(self.data_out(res), *self.cb_args, **self.cb_kwargs)
-            signals.pop_view_state.send(self)
-        elif key == "g":
+        if key == "m_start":
             self.walker.set_focus(0)
-        elif key == "G":
-            self.walker.set_focus(len(self.walker.lst) - 1)
-        elif key in ["h", "left"]:
-            self.walker.left()
-        elif key in ["l", "right"]:
-            self.walker.right()
-        elif key == "tab":
+        elif key == "m_next":
             self.walker.tab_next()
-        elif key == "a":
-            self.walker.add()
-        elif key == "A":
-            self.walker.insert()
-        elif key == "d":
-            self.walker.delete_focus()
+        elif key == "m_end":
+            self.walker.set_focus(len(self.walker.lst) - 1)
+        elif key == "left":
+            self.walker.left()
+        elif key == "right":
+            self.walker.right()
         elif column.keypress(key, self) and not self.handle_key(key):
             return self._w.keypress(size, key)
 
-    def data_out(self, data: Sequence[list]) -> Any:
+    def data_out(self, data: typing.Sequence[list]) -> typing.Any:
         """
             Called on raw list data, before data is returned through the
             callback.
         """
         return data
 
-    def data_in(self, data: Any) -> Iterable[list]:
+    def data_in(self, data: typing.Any) -> typing.Iterable[list]:
         """
             Called to prepare provided data.
         """
         return data
 
-    def is_error(self, col: int, val: Any) -> Optional[str]:
+    def is_error(self, col: int, val: typing.Any) -> typing.Optional[str]:
         """
             Return None, or a string error message.
         """
-        return False
+        return None
 
     def handle_key(self, key):
         return False
 
-    def make_help(self):
-        text = [
-            urwid.Text([("text", "Editor control:\n")])
-        ]
-        keys = [
-            ("A", "insert row before cursor"),
-            ("a", "add row after cursor"),
-            ("d", "delete row"),
-            ("e", "spawn external editor on current field"),
-            ("q", "save changes and exit editor"),
-            ("r", "read value from file"),
-            ("R", "read unescaped value from file"),
-            ("esc", "save changes and exit editor"),
-            ("tab", "next field"),
-            ("enter", "edit field"),
-        ]
-        text.extend(
-            common.format_keyvals(keys, key="key", val="text", indent=4)
+    def cmd_add(self):
+        self.walker.add()
+
+    def cmd_insert(self):
+        self.walker.insert()
+
+    def cmd_delete(self):
+        self.walker.delete_focus()
+
+    def cmd_read_file(self, path):
+        self.walker.set_current_value(read_file(path, False))
+
+    def cmd_read_file_escaped(self, path):
+        self.walker.set_current_value(read_file(path, True))
+
+    def cmd_spawn_editor(self):
+        o = self.walker.get_current_value()
+        if o is not None:
+            n = self.master.spawn_editor(o)
+            n = strutils.clean_hanging_newline(n)
+            self.walker.set_current_value(n)
+
+
+class GridEditor(BaseGridEditor):
+    title = None  # type: str
+    columns = None  # type: typing.Sequence[Column]
+    keyctx = "grideditor"
+
+    def __init__(
+            self,
+            master: "mitmproxy.tools.console.master.ConsoleMaster",
+            value: typing.Any,
+            callback: typing.Callable[..., None],
+            *cb_args,
+            **cb_kwargs
+    ) -> None:
+        super().__init__(
+            master,
+            self.title,
+            self.columns,
+            value,
+            callback,
+            *cb_args,
+            **cb_kwargs
         )
-        text.append(
-            urwid.Text(
-                [
-                    "\n",
-                    ("text", "Values are escaped Python-style strings.\n"),
-                ]
+
+
+class FocusEditor(urwid.WidgetWrap, layoutwidget.LayoutWidget):
+    """
+        A specialised GridEditor that edits the current focused flow.
+    """
+    keyctx = "grideditor"
+
+    def __init__(self, master):
+        self.master = master
+
+    def call(self, v, name, *args, **kwargs):
+        f = getattr(v, name, None)
+        if f:
+            f(*args, **kwargs)
+
+    def get_data(self, flow):
+        """
+            Retrieve the data to edit from the current flow.
+        """
+        raise NotImplementedError
+
+    def set_data(self, vals, flow):
+        """
+            Set the current data on the flow.
+        """
+        raise NotImplementedError
+
+    def set_data_update(self, vals, flow):
+        self.set_data(vals, flow)
+        signals.flow_change.send(self, flow = flow)
+
+    def key_responder(self):
+        return self._w
+
+    def layout_popping(self):
+        self.call(self._w, "layout_popping")
+
+    def layout_pushed(self, prev):
+        if self.master.view.focus.flow:
+            self._w = BaseGridEditor(
+                self.master,
+                self.title,
+                self.columns,
+                self.get_data(self.master.view.focus.flow),
+                self.set_data_update,
+                self.master.view.focus.flow,
             )
-        )
-        return text
+        else:
+            self._w = urwid.Pile([])
