@@ -5,7 +5,6 @@ import os.path
 import re
 from io import BytesIO
 
-import mitmproxy.addons.view
 import mitmproxy.flow
 import tornado.escape
 import tornado.web
@@ -17,6 +16,9 @@ from mitmproxy import http
 from mitmproxy import io
 from mitmproxy import log
 from mitmproxy import version
+from mitmproxy import optmanager
+from mitmproxy.tools.cmdline import CONFIG_PATH
+import mitmproxy.tools.web.master # noqa
 
 
 def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
@@ -41,6 +43,8 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
             continue
         f[conn]["alpn_proto_negotiated"] = \
             f[conn]["alpn_proto_negotiated"].decode(errors="backslashreplace")
+    # There are some bytes in here as well, let's skip it until we have them in the UI.
+    f["client_conn"].pop("tls_extensions", None)
     if flow.error:
         f["error"] = flow.error.get_state()
 
@@ -65,6 +69,7 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
                 "timestamp_start": flow.request.timestamp_start,
                 "timestamp_end": flow.request.timestamp_end,
                 "is_replay": flow.request.is_replay,
+                "pretty_host": flow.request.pretty_host,
             }
         if flow.response:
             if flow.response.raw_content:
@@ -85,6 +90,7 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
                 "is_replay": flow.response.is_replay,
             }
     f.get("server_conn", {}).pop("cert", None)
+    f.get("client_conn", {}).pop("mitmcert", None)
 
     return f
 
@@ -119,7 +125,7 @@ class RequestHandler(tornado.web.RequestHandler):
         self.add_header(
             "Content-Security-Policy",
             "default-src 'self'; "
-            "connect-src 'self' ws://* ; "
+            "connect-src 'self' ws:; "
             "style-src   'self' 'unsafe-inline'"
         )
 
@@ -144,7 +150,7 @@ class RequestHandler(tornado.web.RequestHandler):
             return self.request.body
 
     @property
-    def view(self) -> mitmproxy.addons.view.View:
+    def view(self) -> "mitmproxy.addons.view.View":
         return self.application.master.view
 
     @property
@@ -228,7 +234,8 @@ class DumpFlows(RequestHandler):
     def post(self):
         self.view.clear()
         bio = BytesIO(self.filecontents)
-        self.master.load_flows(io.FlowReader(bio))
+        for i in io.FlowReader(bio).stream():
+            self.master.load_flow(i)
         bio.close()
 
 
@@ -242,7 +249,7 @@ class ResumeFlows(RequestHandler):
     def post(self):
         for f in self.view:
             f.resume()
-            self.view.update(f)
+            self.view.update([f])
 
 
 class KillFlows(RequestHandler):
@@ -250,27 +257,27 @@ class KillFlows(RequestHandler):
         for f in self.view:
             if f.killable:
                 f.kill()
-                self.view.update(f)
+                self.view.update([f])
 
 
 class ResumeFlow(RequestHandler):
     def post(self, flow_id):
         self.flow.resume()
-        self.view.update(self.flow)
+        self.view.update([self.flow])
 
 
 class KillFlow(RequestHandler):
     def post(self, flow_id):
         if self.flow.killable:
             self.flow.kill()
-            self.view.update(self.flow)
+            self.view.update([self.flow])
 
 
 class FlowHandler(RequestHandler):
     def delete(self, flow_id):
         if self.flow.killable:
             self.flow.kill()
-        self.view.remove(self.flow)
+        self.view.remove([self.flow])
 
     def put(self, flow_id):
         flow = self.flow
@@ -313,13 +320,13 @@ class FlowHandler(RequestHandler):
         except APIError:
             flow.revert()
             raise
-        self.view.update(flow)
+        self.view.update([flow])
 
 
 class DuplicateFlow(RequestHandler):
     def post(self, flow_id):
         f = self.flow.copy()
-        self.view.add(f)
+        self.view.add([f])
         self.write(f.id)
 
 
@@ -327,14 +334,14 @@ class RevertFlow(RequestHandler):
     def post(self, flow_id):
         if self.flow.modified():
             self.flow.revert()
-            self.view.update(self.flow)
+            self.view.update([self.flow])
 
 
 class ReplayFlow(RequestHandler):
     def post(self, flow_id):
         self.flow.backup()
         self.flow.response = None
-        self.view.update(self.flow)
+        self.view.update([self.flow])
 
         try:
             self.master.replay_request(self.flow)
@@ -347,7 +354,7 @@ class FlowContent(RequestHandler):
         self.flow.backup()
         message = getattr(self.flow, message)
         message.content = self.filecontents
-        self.view.update(self.flow)
+        self.view.update([self.flow])
 
     def get(self, flow_id, message):
         message = getattr(self.flow, message)
@@ -404,9 +411,10 @@ class Settings(RequestHandler):
         self.write(dict(
             version=version.VERSION,
             mode=str(self.master.options.mode),
+            intercept_active=self.master.options.intercept_active,
             intercept=self.master.options.intercept,
             showhost=self.master.options.showhost,
-            no_upstream_cert=self.master.options.no_upstream_cert,
+            upstream_cert=self.master.options.upstream_cert,
             rawtcp=self.master.options.rawtcp,
             http2=self.master.options.http2,
             websocket=self.master.options.websocket,
@@ -418,12 +426,13 @@ class Settings(RequestHandler):
             contentViews=[v.name.replace(' ', '_') for v in contentviews.views],
             listen_host=self.master.options.listen_host,
             listen_port=self.master.options.listen_port,
+            server=self.master.options.server,
         ))
 
     def put(self):
         update = self.json
         option_whitelist = {
-            "intercept", "showhost", "no_upstream_cert",
+            "intercept", "showhost", "upstream_cert",
             "rawtcp", "http2", "websocket", "anticache", "anticomp",
             "stickycookie", "stickyauth", "stream_large_bodies"
         }
@@ -433,15 +442,35 @@ class Settings(RequestHandler):
         self.master.options.update(**update)
 
 
+class Options(RequestHandler):
+    def get(self):
+        self.write(optmanager.dump_dicts(self.master.options))
+
+    def put(self):
+        update = self.json
+        try:
+            self.master.options.update(**update)
+        except Exception as err:
+            raise APIError(400, "{}".format(err))
+
+
+class SaveOptions(RequestHandler):
+    def post(self):
+        try:
+            optmanager.save(self.master.options, CONFIG_PATH, True)
+        except Exception as err:
+            raise APIError(400, "{}".format(err))
+
+
 class Application(tornado.web.Application):
     def __init__(self, master, debug):
         self.master = master
         handlers = [
             (r"/", IndexHandler),
-            (r"/filter-help", FilterHelp),
+            (r"/filter-help(?:\.json)?", FilterHelp),
             (r"/updates", ClientConnection),
-            (r"/events", Events),
-            (r"/flows", Flows),
+            (r"/events(?:\.json)?", Events),
+            (r"/flows(?:\.json)?", Flows),
             (r"/flows/dump", DumpFlows),
             (r"/flows/resume", ResumeFlows),
             (r"/flows/kill", KillFlows),
@@ -451,12 +480,14 @@ class Application(tornado.web.Application):
             (r"/flows/(?P<flow_id>[0-9a-f\-]+)/duplicate", DuplicateFlow),
             (r"/flows/(?P<flow_id>[0-9a-f\-]+)/replay", ReplayFlow),
             (r"/flows/(?P<flow_id>[0-9a-f\-]+)/revert", RevertFlow),
-            (r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response)/content", FlowContent),
+            (r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response)/content.data", FlowContent),
             (
-                r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response)/content/(?P<content_view>[0-9a-zA-Z\-\_]+)",
+                r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response)/content/(?P<content_view>[0-9a-zA-Z\-\_]+)(?:\.json)?",
                 FlowContentView),
-            (r"/settings", Settings),
+            (r"/settings(?:\.json)?", Settings),
             (r"/clear", ClearAll),
+            (r"/options(?:\.json)?", Options),
+            (r"/options/save", SaveOptions)
         ]
         settings = dict(
             template_path=os.path.join(os.path.dirname(__file__), "templates"),
